@@ -1362,6 +1362,7 @@ void
 gaim_accounts_sync(void)
 {
 	FILE *fp;
+	struct stat st;
 	const char *user_dir = gaim_user_dir();
 	char *filename;
 	char *filename_real;
@@ -1378,20 +1379,20 @@ gaim_accounts_sync(void)
 
 	gaim_debug(GAIM_DEBUG_INFO, "accounts", "Writing accounts to disk.\n");
 
-	fp = fopen(user_dir, "r");
+	fp = g_fopen(user_dir, "r");
 
 	if (fp == NULL)
-		mkdir(user_dir, S_IRUSR | S_IWUSR | S_IXUSR);
+		g_mkdir(user_dir, S_IRUSR | S_IWUSR | S_IXUSR);
 	else
 		fclose(fp);
 
 	filename = g_build_filename(user_dir, "accounts.xml.save", NULL);
 
-	if ((fp = fopen(filename, "w")) != NULL) {
+	if ((fp = g_fopen(filename, "w")) != NULL) {
 		GList *l;
 
 		fprintf(fp, "<?xml version='1.0' encoding='UTF-8' ?>\n\n");
-		fprintf(fp, "<accounts>\n");
+		fprintf(fp, "<accounts version='1.0'>\n");
 
 		for (l = gaim_accounts_get_all(); l != NULL; l = l->next)
 			gaim_accounts_write(fp, l->data);
@@ -1408,9 +1409,16 @@ gaim_accounts_sync(void)
 		return;
 	}
 
+	if (g_stat(filename, &st) || (st.st_size == 0)) {
+		gaim_debug_error("accounts", "Failed to save accounts\n");
+		g_unlink(filename);
+		g_free(filename);
+		return;
+	}
+
 	filename_real = g_build_filename(user_dir, "accounts.xml", NULL);
 
-	if (rename(filename, filename_real) < 0) {
+	if (g_rename(filename, filename_real) < 0) {
 		gaim_debug(GAIM_DEBUG_ERROR, "accounts", "Error renaming %s to %s\n",
 				   filename, filename_real);
 	}
@@ -1459,15 +1467,23 @@ gaim_accounts_delete(GaimAccount *account)
 	for (gnode = gaim_get_blist()->root; gnode != NULL; gnode = gnode->next) {
 		if (!GAIM_BLIST_NODE_IS_GROUP(gnode))
 			continue;
-		for (cnode = gnode->child; cnode; cnode = cnode->next) {
+
+		cnode = gnode->child;
+		while (cnode) {
+			GaimBlistNode *cnode_next = cnode->next;
+
 			if(GAIM_BLIST_NODE_IS_CONTACT(cnode)) {
-				for (bnode = cnode->child; bnode; bnode = bnode->next) {
+				bnode = cnode->child;
+				while (bnode) {
+					GaimBlistNode *bnode_next = bnode->next;
+
 					if (GAIM_BLIST_NODE_IS_BUDDY(bnode)) {
 						GaimBuddy *b = (GaimBuddy *)bnode;
 
 						if (b->account == account)
 							gaim_blist_remove_buddy(b);
 					}
+					bnode = bnode_next;
 				}
 			} else if (GAIM_BLIST_NODE_IS_CHAT(cnode)) {
 				GaimChat *c = (GaimChat *)cnode;
@@ -1475,6 +1491,7 @@ gaim_accounts_delete(GaimAccount *account)
 				if (c->account == account)
 					gaim_blist_remove_chat(c);
 			}
+			cnode = cnode_next;
 		}
 	}
 
