@@ -246,7 +246,7 @@ screenname_changed_cb(GtkEntry *entry, AccountPrefsDialog *dialog)
 static void
 icon_filesel_choose_cb(GtkWidget *widget, gint response, AccountPrefsDialog *dialog)
 {
-	const char *filename;
+	char *filename;
 
 	if (response != GTK_RESPONSE_ACCEPT) {
 		if (response == GTK_RESPONSE_CANCEL)
@@ -260,15 +260,16 @@ icon_filesel_choose_cb(GtkWidget *widget, gint response, AccountPrefsDialog *dia
 static void
 icon_filesel_choose_cb(GtkWidget *w, AccountPrefsDialog *dialog)
 {
-	const char *filename;
+	char *filename;
 
-	filename = gtk_file_selection_get_filename(
-			GTK_FILE_SELECTION(dialog->icon_filesel));
+	filename = g_strdup(gtk_file_selection_get_filename(
+			GTK_FILE_SELECTION(dialog->icon_filesel)));
 
 	/* If they typed in a directory, change there */
 	if (gaim_gtk_check_if_dir(filename,
 			GTK_FILE_SELECTION(dialog->icon_filesel)))
 	{
+		g_free(filename);
 		return;
 	}
 #endif /* FILECHOOSER */
@@ -281,6 +282,7 @@ icon_filesel_choose_cb(GtkWidget *w, AccountPrefsDialog *dialog)
 
 	gtk_widget_destroy(dialog->icon_filesel);
 	dialog->icon_filesel = NULL;
+	g_free(filename);
  }
 
 static void
@@ -294,18 +296,21 @@ icon_preview_change_cb(GtkTreeSelection *sel, AccountPrefsDialog *dialog)
 	int height, width;
 	char *basename, *markup, *size;
 	struct stat st;
-	const char *filename;
+	char *filename;
 
 #if GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
 	filename = gtk_file_chooser_get_preview_filename(
 					GTK_FILE_CHOOSER(dialog->icon_filesel));
 #else /* FILECHOOSER */
-	filename = gtk_file_selection_get_filename(
-		GTK_FILE_SELECTION(dialog->icon_filesel));
+	filename = g_strdup(gtk_file_selection_get_filename(
+		GTK_FILE_SELECTION(dialog->icon_filesel)));
 #endif /* FILECHOOSER */
 
-	if (!filename || stat(filename, &st))
+	if (!filename || g_stat(filename, &st))
+	{
+		g_free(filename);
 		return;
+	}
 
 	pixbuf = gdk_pixbuf_new_from_file(filename, NULL);
 	if (!pixbuf) {
@@ -315,6 +320,7 @@ icon_preview_change_cb(GtkTreeSelection *sel, AccountPrefsDialog *dialog)
 		gtk_file_chooser_set_preview_widget_active(
 					GTK_FILE_CHOOSER(dialog->icon_filesel), FALSE);
 #endif /* FILECHOOSER */
+		g_free(filename);
 		return;
 	}
 
@@ -338,6 +344,7 @@ icon_preview_change_cb(GtkTreeSelection *sel, AccountPrefsDialog *dialog)
 
 	g_object_unref(G_OBJECT(pixbuf));
 	g_object_unref(G_OBJECT(scale));
+	g_free(filename);
 	g_free(basename);
 	g_free(size);
 	g_free(markup);
@@ -515,7 +522,7 @@ convert_buddy_icon(GaimPlugin *plugin, const char *path)
 	format = gdk_pixbuf_get_file_info (path, &width, &height);
 #else
 	loader = gdk_pixbuf_loader_new();
-	if (!stat(path, &st) && (file = fopen(path, "rb")) != NULL) {
+	if (!g_stat(path, &st) && (file = g_fopen(path, "rb")) != NULL) {
 		data = g_malloc(st.st_size);
 		fread(data, 1, st.st_size, file);
 		fclose(file);
@@ -537,6 +544,8 @@ convert_buddy_icon(GaimPlugin *plugin, const char *path)
 		   prpl_info->icon_spec.max_width >= width &&
 		   prpl_info->icon_spec.min_height <= height &&
 		   prpl_info->icon_spec.max_height >= height))) {                                  /* The icon is the correct size */
+		g_strfreev(prpl_formats);
+		g_strfreev(pixbuf_formats);
 #endif
 		return g_strdup(path);
 #if GTK_CHECK_VERSION(2,2,0)
@@ -548,6 +557,7 @@ convert_buddy_icon(GaimPlugin *plugin, const char *path)
 		const char *dirname = gaim_buddy_icons_get_cache_dir();
 		char *filename = g_build_filename(dirname, random, NULL);
 		pixbuf = gdk_pixbuf_new_from_file(path, &error);
+		g_strfreev(pixbuf_formats);
 		if (!error && prpl_info->icon_spec.scale_rules & GAIM_ICON_SCALE_SEND) {
 			int new_width = gdk_pixbuf_get_width(pixbuf);
 			int new_height = gdk_pixbuf_get_height(pixbuf);
@@ -571,16 +581,18 @@ convert_buddy_icon(GaimPlugin *plugin, const char *path)
 			g_free(random);
 			gaim_debug_error("buddyicon", "Could not open icon for conversion: %s\n", error->message);
 			g_error_free(error);
+			g_strfreev(prpl_formats);
 			return NULL;
 		}
 
 		if (!g_file_test(dirname, G_FILE_TEST_IS_DIR)) {
 			gaim_debug_info("buddyicon", "Creating icon cache directory.\n");
 
-			if (mkdir(dirname, S_IRUSR | S_IWUSR | S_IXUSR) < 0) {
+			if (g_mkdir(dirname, S_IRUSR | S_IWUSR | S_IXUSR) < 0) {
 				gaim_debug_error("buddyicon",
 								 "Unable to create directory %s: %s\n",
 								 dirname, strerror(errno));
+				g_strfreev(prpl_formats);
 				return NULL;
 			}
 		}
@@ -592,6 +604,7 @@ convert_buddy_icon(GaimPlugin *plugin, const char *path)
 			if (gdk_pixbuf_save (pixbuf, filename, prpl_formats[i], &error, NULL) == TRUE)
 					break;
 		}
+		g_strfreev(prpl_formats);
 		if (!error) {
 			g_free(random);
 			g_object_unref(G_OBJECT(pixbuf));
@@ -1293,6 +1306,9 @@ account_win_destroy_cb(GtkWidget *w, GdkEvent *event,
 	if (dialog->protocol_id != NULL)
 		g_free(dialog->protocol_id);
 
+	if (dialog->icon_path != NULL)
+		g_free(dialog->icon_path);
+
 	if (dialog->icon_filesel)
 		gtk_widget_destroy(dialog->icon_filesel);
 
@@ -2005,7 +2021,7 @@ ask_delete_account_sel(GtkTreeModel *model, GtkTreePath *path,
 							  gaim_account_get_username(account));
 
 		gaim_request_close_with_handle(account);
-		gaim_request_action(account, NULL, buf, NULL, 1, account, 2,
+		gaim_request_action(account, NULL, buf, NULL, 0, account, 2,
 							_("Delete"), delete_account_cb,
 							_("Cancel"), NULL);
 		g_free(buf);
@@ -2501,7 +2517,7 @@ gaim_gtk_accounts_notify_added(GaimAccount *account, const char *remote_user,
 	else
 	{
 		gaim_request_action(NULL, NULL, _("Add buddy to your list?"),
-							buffer, 0, data, 2,
+							buffer, GAIM_DEFAULT_ACTION_NONE, data, 2,
 							_("Add"),    G_CALLBACK(add_user_cb),
 							_("Cancel"), G_CALLBACK(free_add_user_data));
 	}
