@@ -60,19 +60,33 @@ struct translator {
 };
 
 static struct developer developers[] = {
-	{"Rob Flynn",					N_("maintainer"), "gaim@robflynn.com"},
 	{"Sean Egan",					N_("lead developer"), "sean.egan@binghamton.edu"},
+	{"Daniel 'datallah' Atallah",	N_("developer"),	NULL},
 	{"Christian 'ChipX86' Hammond",	N_("developer & webmaster"), NULL},
-	{"Herman Bloggs",				N_("win32 port"), "hermanator12002@yahoo.com"},
+	{"Herman Bloggs",				N_("win32 port"), "herman@bluedigits.com"},
 	{"Nathan 'faceprint' Walp",		N_("developer"), NULL},
 	{"Mark 'KingAnt' Doliner",		N_("developer"), NULL},
 	{"Ethan 'Paco-Paco' Blanton",	N_("developer"), NULL},
+	{"Etan 'deryni' Reisner",       N_("developer"), NULL},
 	{"Tim 'marv' Ringenbach",		N_("developer"), NULL},
 	{"Luke 'LSchiere' Schierer",	N_("support"), NULL},
+	{"Stu 'nosnilmot' Tomlinson",	N_("developer"), NULL},
+	{NULL, NULL, NULL}
+};
+
+static struct developer patch_writers[] = {
+	{"Ka-Hing 'javabsp' Cheung",	NULL,	NULL},
+	{"Felipe 'shx' Contreras",		NULL,	NULL},
+	{"Decklin Foster",				NULL,	NULL},
+	{"Gary 'grim' Kramlich",		NULL,	NULL},
+	{"Robert 'Robot101' McQueen",	NULL,	NULL},
+	{"Benjamin Miller",				NULL,	NULL},
+	{"Kevin 'SimGuy' Stange",		NULL,	NULL},
 	{NULL, NULL, NULL}
 };
 
 static struct developer retired_developers[] = {
+	{"Rob Flynn",					N_("maintainer"), "gaim@robflynn.com"},
 	{"Adam Fritzler",		N_("former libfaim maintainer"), NULL},
 	{"Eric Warmenhoven",	N_("former lead developer"), "warmenhoven@yahoo.com"},
 	{"Jim Duchek",			N_("former maintainer"), NULL},
@@ -84,13 +98,13 @@ static struct developer retired_developers[] = {
 
 static struct translator current_translators[] = {
 	{N_("Bulgarian"),			"bg", "Alexander Shopov", "al_shopov@users.sf.net"},
-	{N_("Catalan"),				"ca", "Robert Millan", "zeratul2@wanadoo.es"},
+	{N_("Catalan"),				"ca", "Josep Puigdemont", "tradgnome@softcatala.org"},
 	{N_("Czech"),				"cs", "Miloslav Trmac", "mitr@volny.cz"},
 	{N_("Danish"),				"da", "Morten Brix Pedersen", "morten@wtf.dk"},
-	{N_("Australian English"), "en_AU", "Peter (Bleeter) Lawler", "trans@six-by-nine.com.au"},
-	{N_("British English"),		"en_GB", "Luke Ross", "lukeross@sys3175.co.uk"},
-	{N_("Canadian English"),	"en_CA", "Adam Weinberger", "adamw@gnome.org"},
 	{N_("German"),				"de", "Björn Voigt", "bjoern@cs.tu-berlin.de"},
+	{N_("Australian English"),	"en_AU", "Peter Lawler", "trans@six-by-nine.com.au"},
+	{N_("Canadian English"),	"en_CA", "Adam Weinberger", "adamw@gnome.org"},
+	{N_("British English"),		"en_GB", "Luke Ross", "lukeross@sys3175.co.uk"},
 	{N_("Spanish"),				"es", "Javier Fernández-Sanguino Peña", "jfs@debian.org"},
 	{N_("Finnish"),				"fi", "Arto Alakulju", "arto@alakulju.net"},
 	{N_("French"),				"fr", "Éric Boumaour", "zongo_fr@users.sourceforge.net"},
@@ -99,19 +113,23 @@ static struct translator current_translators[] = {
 	{N_("Hungarian"),			"hu", "Zoltan Sutto", "suttozoltan@chello.hu"},
 	{N_("Italian"),				"it", "Claudio Satriano", "satriano@na.infn.it"},
 	{N_("Japanese"),			"ja", "Takashi Aihana", "aihana@gnome.gr.jp"},
-	{N_("Lithuanian"),			"lt", "Gediminas Čičinskas", "gediminas@parok.lt"},
 	{N_("Korean"),				"ko", "Kyung-uk Son", "vvs740@chol.com"},
-	{N_("Dutch; Flemish"),		"nl", "Vincent van Adrighem", "V.vanAdrighem@dirck.mine.nu"},
+	{N_("Lithuanian"),			"lt", "Gediminas Čičinskas", "gediminas@parok.lt"},
 	{N_("Macedonian"),			"mk", "Tomislav Markovski", "herrera@users.sf.net"},
+	{N_("Burmese"),             "my_MM", "Minn Myat Soe", "mmyatsoe@gmail.com"},
+	{N_("Dutch; Flemish"),		"nl", "Vincent van Adrighem", "V.vanAdrighem@dirck.mine.nu"},
 	{N_("Norwegian"),			"no", "Petter Johan Olsen", "petter.olsen@cc.uit.no"},
 	{N_("Polish"),				"pl", "Krzysztof Foltman, Emil Nowak", "krzysztof@foltman.com;emil5@go2.pl"},
 	{N_("Portuguese"),			"pt", "Duarte Henriques", "duarte_henriques@myrealbox.com"},
 	{N_("Portuguese-Brazil"),	"pt_BR", "Maurício de Lemos Rodrigues Collares Neto", "mauricioc@gmail.com"},
 	{N_("Romanian"),			"ro", "Mişu Moldovan", "dumol@go.ro"},
 	{N_("Russian"),				"ru", "Dmitry Beloglazov", "dmaa@users.sf.net"},
-	{N_("Serbian"),				"sr", "Danilo Šegan, Aleksandar Urosevic", "dsegan@gmx.net;urke@users.sourceforge.net"},
 	{N_("Slovenian"),			"sl", "Martin Srebotnjak", "miles@filmsi.net"},
-	{N_("Swedish"),				"sv", "Tore Lundqvist", "tlt@mima.x.se"},
+	{N_("Albanian"),            "sq", "Programe Shqip", "besnik@spymac.com"},
+	{N_("Serbian"),				"sr", "Danilo Šegan, Aleksandar Urosevic", "dsegan@gmx.net;urke@users.sourceforge.net"},
+	{N_("Swedish"),				"sv", "Peter Hjalmarsson", "xake@telia.com"},
+	{N_("Turkish"),             "tr", "Onur Kucuktunc", "onurcc@gmail.com"},
+	{N_("Ukrainian"),           "uk", "Maxim V. Dziumanenko", "mvd@mylinux.com.ua"},
 	{N_("Vietnamese"),			"vi", N_("T.M.Thanh and the Gnome-Vi Team"), "gnomevi-list@lists.sf.net"},
 	{N_("Simplified Chinese"),	"zh_CN", "Funda Wang", "fundawang@linux.net.cn"},
 	{N_("Traditional Chinese"),	"zh_TW", "Ambrose C. Li, Paladin R. Liu", "acli@ada.dhs.org;paladin@ms1.hinet.net"},
@@ -123,6 +141,7 @@ static struct translator past_translators[] = {
 	{N_("Amharic"),				"am", "Daniel Yacob", NULL},
 	{N_("Bulgarian"),			"bg", "Hristo Todorov", NULL},
 	{N_("Catalan"),				"ca", "JM Pérez Cáncer", NULL},
+	{N_("Catalan"),				"ca", "Robert Millan", NULL},
 	{N_("Czech"),				"cs", "Honza Král", NULL},
 	{N_("German"),				"de", "Daniel Seifert, Karsten Weiss", NULL},
 	{N_("Spanish"),				"es", "Amaya Rodrigo, Alejandro G Villar, Nicolás Lichtmaier, JM Pérez Cáncer", NULL},
@@ -135,6 +154,7 @@ static struct translator past_translators[] = {
 	{N_("Russian"),				"ru", "Sergey Volozhanin", NULL},
 	{N_("Russian"),				"ru", "Alexandre Prokoudine", NULL},
 	{N_("Slovak"),				"sk", "Daniel Režný", NULL},
+	{N_("Swedish"),				"sv", "Tore Lundqvist", "tlt@mima.x.se"},
 	{N_("Swedish"),				"sv", "Christian Rose", NULL},
 	{N_("Chinese"),				"zh_CN, zh_TW", "Hashao, Rocky S. Lee", NULL},
 	{NULL, NULL, NULL, NULL}
@@ -209,6 +229,7 @@ void gaim_gtkdialogs_about(GtkWidget *w, void *data)
 
 	text = gtk_imhtml_new(NULL, NULL);
 	gtk_container_add(GTK_CONTAINER(sw), text);
+	gtk_imhtml_set_format_functions(GTK_IMHTML(text), GTK_IMHTML_ALL ^ GTK_IMHTML_SMILEY);
 	gaim_setup_imhtml(text);
 
 	str = g_string_sized_new(4096);
@@ -244,18 +265,17 @@ void gaim_gtkdialogs_about(GtkWidget *w, void *data)
 	/* Crazy Patch Writers */
 	g_string_append_printf(str, "<FONT SIZE=\"4\">%s:</FONT><BR/>",
 						   _("Crazy Patch Writers"));
-	g_string_append(str,
-					"  Daniel 'datallah' Atallah<BR>"
-					"  Ka-Hing 'javabsp' Cheung<BR>"
-					"  Felipe 'shx' Contreras<BR>"
-					"  Decklin Foster<BR>"
-					"  Gary 'grim' Kramlich<BR>"
-					"  Robert 'Robot101' McQueen<BR>"
-					"  Benjamin Miller<BR>"
-					"  Etan 'deryni' Reisner<BR>"
-					"  Kevin 'SimGuy' Stange<BR>"
-					"  Stu 'nosnilmot' Tomlinson<BR>"
-					"<BR>");
+	for (i = 0; patch_writers[i].name != NULL; i++) {
+		if (patch_writers[i].email != NULL) {
+			g_string_append_printf(str, "  %s &lt;<a href=\"mailto:%s\">%s</a>&gt;<br/>",
+					patch_writers[i].name,
+					patch_writers[i].email, patch_writers[i].email);
+		} else {
+			g_string_append_printf(str, "  %s<br/>",
+					patch_writers[i].name);
+		}
+	}
+	g_string_append(str, "<BR/>");
 
 	/* Retired Developers */
 	g_string_append_printf(str, "<FONT SIZE=\"4\">%s:</FONT><BR/>",
@@ -312,6 +332,90 @@ void gaim_gtkdialogs_about(GtkWidget *w, void *data)
 	}
 	g_string_append(str, "<BR/>");
 
+	/* The following primarly intented for user/developer interaction and thus
+	   ought not be translated */
+	g_string_append(str, "<b>Gaim was compiled with the following:</b><br/>");
+#ifdef CONFIG_ARGS /* win32 build doesn't use configure */
+	g_string_append(str, "Arguments to <i>./configure</i>:  " CONFIG_ARGS "<br/>");
+#endif
+#ifdef DEBUG
+	g_string_append(str, "Print debugging messages:  Yes<br/>");
+#else
+	g_string_append(str, "Print debugging messages:  No<br/>");
+#endif /* DEBUG */
+#ifdef ENABLE_BINRELOC
+	g_string_append(str, "Binary relocation:  Enabled<br/>");
+#else
+	g_string_append(str, "Binary relocation:  Disabled<br/>");
+#endif /* ENABLE_BINRELOC */
+#ifdef GAIM_PLUGINS
+	g_string_append(str, "Plugins:  Enabled<br/>");
+#else
+	g_string_append(str, "Plugins:  Disabled<br/>");
+#endif /* GAIM_PLUGINS */
+#ifdef HAVE_SSL
+	g_string_append(str, "SSL:  Gaim was compiled with SSL support<br/>");
+#else
+	g_string_append(str, "SSL:  Gaim was <b><i>NOT</i></b> compiled with any SSL support!<br/>");
+#endif
+#ifdef HAVE_GNUTLS
+	g_string_append(str, "GNUTLS:  Enabled<br/>");
+#else
+	g_string_append(str, "GNUTLS:  Disabled<br/>");
+#endif
+#ifdef HAVE_NSS
+	g_string_append(str, "NSS:  Enabled<br/>");
+#else
+	g_string_append(str, "NSS:  Disabled<br/>");
+#endif
+#ifdef HAVE_TCL
+	g_string_append(str, "Tcl: Yes<br/>");
+#else
+	g_string_append(str, "Tcl: No<br/>");
+#endif
+#ifdef HAVE_TK
+	g_string_append(str, "Tk:  Yes<br/>");
+#else
+	g_string_append(str, "Tk:  No<br/>");
+#endif
+#ifdef LIBZEPHYR_EXT
+	g_string_append(str, "External libzephyr:  Yes<br/>");
+#else
+	g_string_append(str, "External libzephyr:  No<br/>");
+#endif
+#ifdef ZEPHYR_USES_KERBEROS
+	g_string_append(str, "Zephyr uses Kerberos: Yes<br/>");
+#else
+	g_string_append(str, "Zephyr uses Kerberos: No<br/>");
+#endif
+#ifdef USE_AO
+	g_string_append(str, "AO:  Yes<br/>");
+#else
+	g_string_append(str, "AO:  No<br/>");
+#endif
+#ifdef USE_NAS_AUDIO
+	g_string_append(str, "NAS Audio:  Yes<br/>");
+#else
+	g_string_append(str, "NAS Audio:  No<br/>");
+#endif
+#ifdef USE_GTKSPELL
+	g_string_append(str, "GtkSpell:  Enabled<br/>");
+#else
+	g_string_append(str, "GtkSpell:  Disabled<br/>");
+#endif
+#ifdef USE_SCREENSAVER
+	g_string_append(str, "XScreenSaver support:  Yes<br/>");
+#else
+	g_string_append(str, "XScreenSaver support:  No<br/>");
+#endif
+#ifdef USE_SM
+	g_string_append(str, "X SM support:  Yes<br/>");
+#else
+	g_string_append(str, "X SM support:  No<br/>");
+#endif
+
+	/* End of not to be translated section */
+
 	gtk_imhtml_append_text(GTK_IMHTML(text), str->str, GTK_IMHTML_NO_SCROLL);
 	g_string_free(str, TRUE);
 
@@ -754,8 +858,8 @@ gaim_gtkdialogs_remove_contact(GaimContact *contact)
 		gchar *text = g_strdup_printf(_("You are about to remove the contact containing %s and %d other buddies from your buddy list.  Do you want to continue?"),
 			       buddy->name, contact->totalsize - 1);
 
-		gaim_request_action(NULL, NULL, _("Remove Contact"), text, 
-				GAIM_DEFAULT_ACTION_NONE, contact, 2,
+		gaim_request_action(contact, NULL, _("Remove Contact"), text, 
+				0, contact, 2,
 				_("Remove Contact"), G_CALLBACK(gaim_gtkdialogs_remove_contact_cb),
 				_("Cancel"), NULL);
 
@@ -815,7 +919,7 @@ gaim_gtkdialogs_remove_group(GaimGroup *group)
 	text = g_strdup_printf(_("You are about to remove the group %s and all its members from your buddy list.  Do you want to continue?"),
 						   group->name);
 
-	gaim_request_action(NULL, NULL, _("Remove Group"), text, -1, group, 2,
+	gaim_request_action(group, NULL, _("Remove Group"), text, 0, group, 2,
 						_("Remove Group"), G_CALLBACK(gaim_gtkdialogs_remove_group_cb),
 						_("Cancel"), NULL);
 
@@ -857,8 +961,8 @@ gaim_gtkdialogs_remove_buddy(GaimBuddy *buddy)
 	text = g_strdup_printf(_("You are about to remove %s from your buddy list.  Do you want to continue?"),
 						   buddy->name);
 
-	gaim_request_action(NULL, NULL, _("Remove Buddy"), text, 
-						GAIM_DEFAULT_ACTION_NONE, buddy, 2,
+	gaim_request_action(buddy, NULL, _("Remove Buddy"), text, 
+						0, buddy, 2,
 						_("Remove Buddy"), G_CALLBACK(gaim_gtkdialogs_remove_buddy_cb),
 						_("Cancel"), NULL);
 
@@ -869,15 +973,18 @@ static void
 gaim_gtkdialogs_remove_chat_cb(GaimChat *chat)
 {
 	char *name = NULL;
+	GaimAccount *account;
 	GaimConversation *conv = NULL;
 
-	if (GAIM_PLUGIN_PROTOCOL_INFO(chat->account->gc->prpl)->get_chat_name != NULL)
-		name = GAIM_PLUGIN_PROTOCOL_INFO(chat->account->gc->prpl)->get_chat_name(chat->components);
+	account = chat->account;
+
+	if (GAIM_PLUGIN_PROTOCOL_INFO(account->gc->prpl)->get_chat_name != NULL)
+		name = GAIM_PLUGIN_PROTOCOL_INFO(account->gc->prpl)->get_chat_name(chat->components);
 
 	gaim_blist_remove_chat(chat);
 
 	if (name != NULL) {
-		conv = gaim_find_conversation_with_account(name, chat->account);
+		conv = gaim_find_conversation_with_account(name, account);
 		g_free(name);
 	}
 
@@ -893,7 +1000,7 @@ gaim_gtkdialogs_remove_chat(GaimChat *chat)
 
 	g_return_if_fail(chat != NULL);
 
-	gaim_request_action(NULL, NULL, _("Remove Chat"), text, -1, chat, 2,
+	gaim_request_action(chat, NULL, _("Remove Chat"), text, 0, chat, 2,
 						_("Remove Chat"), G_CALLBACK(gaim_gtkdialogs_remove_chat_cb),
 						_("Cancel"), NULL);
 
