@@ -1,7 +1,9 @@
 /*
  * gaim
  *
- * Copyright (C) 1998-1999, Mark Spencer <markster@marko.net>
+ * Gaim is the legal property of its developers, whose names are too numerous
+ * to list here.  Please refer to the COPYRIGHT file distributed with this
+ * source distribution.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +28,7 @@
 #include "prpl.h"
 #include "request.h"
 #include "signals.h"
+#include "version.h"
 
 #ifdef _WIN32
 # define PLUGIN_EXT ".dll"
@@ -71,6 +74,15 @@ static void *load_cb_data = NULL;
 static void (*unload_cb)(GaimPlugin *, void *) = NULL;
 static void *unload_cb_data = NULL;
 
+
+void *
+gaim_plugins_get_handle(void)
+{
+	static int handle;
+	return &handle;
+}
+
+
 #ifdef GAIM_PLUGINS
 static int
 is_so_file(const char *filename, const char *ext)
@@ -133,10 +145,17 @@ static gint
 compare_prpl(GaimPlugin *a, GaimPlugin *b)
 {
 	/* neg if a before b, 0 if equal, pos if a after b */
-	return ((GAIM_IS_PROTOCOL_PLUGIN(a)
-			 ? GAIM_PLUGIN_PROTOCOL_INFO(a)->protocol : -1) -
-			((GAIM_IS_PROTOCOL_PLUGIN(b)
-			 ? GAIM_PLUGIN_PROTOCOL_INFO(b)->protocol : -1)));
+	if(GAIM_IS_PROTOCOL_PLUGIN(a)) {
+		if(GAIM_IS_PROTOCOL_PLUGIN(b))
+			return strcmp(a->info->name, b->info->name);
+		else
+			return -1;
+	} else {
+		if(GAIM_IS_PROTOCOL_PLUGIN(b))
+			return 1;
+		else
+			return 0;
+	}
 }
 
 GaimPlugin *
@@ -174,6 +193,7 @@ gaim_plugin_probe(const char *filename)
 	plugin = gaim_plugin_new(is_so_file(filename, PLUGIN_EXT), filename);
 
 	if (plugin->native_plugin) {
+		gpointer pgaim_init_plugin;
 		const char *error;
 		plugin->handle = g_module_open(filename, 0);
 
@@ -188,7 +208,7 @@ gaim_plugin_probe(const char *filename)
 		}
 
 		if (!g_module_symbol(plugin->handle, "gaim_init_plugin",
-							 (gpointer *)&gaim_init_plugin)) {
+							 &pgaim_init_plugin)) {
 			g_module_close(plugin->handle);
 			plugin->handle = NULL;
 
@@ -200,6 +220,7 @@ gaim_plugin_probe(const char *filename)
 
 			return NULL;
 		}
+		gaim_init_plugin = pgaim_init_plugin;
 	}
 	else {
 		loader = find_loader_for_plugin(plugin);
@@ -221,6 +242,16 @@ gaim_plugin_probe(const char *filename)
 		return NULL;
 	}
 
+	if (plugin->info->magic != GAIM_PLUGIN_MAGIC ||
+			plugin->info->major_version != GAIM_MAJOR_VERSION ||
+			plugin->info->minor_version > GAIM_MINOR_VERSION)
+	{
+		gaim_debug(GAIM_DEBUG_ERROR, "plugins", "%s is unloadable: API version mismatch %d.%d.x (need %d.%d.x)\n",
+				   plugin->path, plugin->info->major_version, plugin->info->minor_version, GAIM_MAJOR_VERSION, GAIM_MINOR_VERSION);
+		gaim_plugin_destroy(plugin);
+		return NULL;
+	}
+
 	return plugin;
 #else
 	return NULL;
@@ -338,6 +369,8 @@ gaim_plugin_load(GaimPlugin *plugin)
 	if (load_cb != NULL)
 		load_cb(plugin, load_cb_data);
 
+	gaim_signal_emit(gaim_plugins_get_handle(), "plugin-load", plugin);
+
 	return TRUE;
 
 #else
@@ -398,7 +431,7 @@ gaim_plugin_unload(GaimPlugin *plugin)
 
 		loader_info = GAIM_PLUGIN_LOADER_INFO(loader);
 
-		if (loader_info->load != NULL)
+		if (loader_info->unload != NULL)
 			loader_info->unload(plugin);
 	}
 
@@ -409,6 +442,9 @@ gaim_plugin_unload(GaimPlugin *plugin)
 	if (unload_cb != NULL)
 		unload_cb(plugin, unload_cb_data);
 
+	/* I suppose this is the right place to call this... */
+	gaim_signal_emit(gaim_plugins_get_handle(), "plugin-unload", plugin);
+
 	return TRUE;
 #else
 	return TRUE;
@@ -448,30 +484,52 @@ gaim_plugin_destroy(GaimPlugin *plugin)
 	if (load_queue != NULL)
 		load_queue = g_list_remove(load_queue, plugin);
 
+	/* true, this may leak a little memory if there is a major version
+	 * mismatch, but it's a lot better than trying to free something
+	 * we shouldn't, and crashing while trying to load an old plugin */
+	if(plugin->info == NULL || plugin->info->magic != GAIM_PLUGIN_MAGIC ||
+			plugin->info->major_version != GAIM_MAJOR_VERSION) {
+		if(plugin->handle)
+			g_module_close(plugin->handle);
+		g_free(plugin);
+		return;
+	}
+
 	if (plugin->info != NULL && plugin->info->dependencies != NULL)
 		g_list_free(plugin->info->dependencies);
 
-	if (plugin->native_plugin) {
-
-		if (plugin->info != NULL && plugin->info->type == GAIM_PLUGIN_LOADER) {
+	if (plugin->native_plugin)
+	{
+		if (plugin->info != NULL && plugin->info->type == GAIM_PLUGIN_LOADER)
+		{
+			GaimPluginLoaderInfo *loader_info;
 			GList *exts, *l, *next_l;
 			GaimPlugin *p2;
 
-			for (exts = GAIM_PLUGIN_LOADER_INFO(plugin)->exts;
-				 exts != NULL;
-				 exts = exts->next) {
+			loader_info = GAIM_PLUGIN_LOADER_INFO(plugin);
 
-				for (l = gaim_plugins_get_all(); l != NULL; l = next_l) {
-					next_l = l->next;
+			if (loader_info != NULL && loader_info->exts != NULL)
+			{
+				for (exts = GAIM_PLUGIN_LOADER_INFO(plugin)->exts;
+					 exts != NULL;
+					 exts = exts->next) {
 
-					p2 = l->data;
+					for (l = gaim_plugins_get_all(); l != NULL; l = next_l)
+					{
+						next_l = l->next;
 
-					if (p2->path != NULL && is_so_file(p2->path, exts->data))
-						gaim_plugin_destroy(p2);
+						p2 = l->data;
+
+						if (p2->path != NULL &&
+							is_so_file(p2->path, exts->data))
+						{
+							gaim_plugin_destroy(p2);
+						}
+					}
 				}
-			}
 
-			g_list_free(GAIM_PLUGIN_LOADER_INFO(plugin)->exts);
+				g_list_free(loader_info->exts);
+			}
 
 			plugin_loaders = g_list_remove(plugin_loaders, plugin);
 		}
@@ -482,13 +540,15 @@ gaim_plugin_destroy(GaimPlugin *plugin)
 		if (plugin->handle != NULL)
 			g_module_close(plugin->handle);
 	}
-	else {
+	else
+	{
 		GaimPlugin *loader;
 		GaimPluginLoaderInfo *loader_info;
 
 		loader = find_loader_for_plugin(plugin);
 
-		if (loader != NULL) {
+		if (loader != NULL)
+		{
 			loader_info = GAIM_PLUGIN_LOADER_INFO(loader);
 
 			if (loader_info->destroy != NULL)
@@ -520,8 +580,13 @@ destroy_ipc_info(void *data)
 	GaimPluginIpcCommand *ipc_command = (GaimPluginIpcCommand *)data;
 	int i;
 
-	for (i = 0; i < ipc_command->num_params; i++)
-		gaim_value_destroy(ipc_command->params[i]);
+	if (ipc_command->params != NULL)
+	{
+		for (i = 0; i < ipc_command->num_params; i++)
+			gaim_value_destroy(ipc_command->params[i]);
+
+		g_free(ipc_command->params);
+	}
 
 	if (ipc_command->ret_value != NULL)
 		gaim_value_destroy(ipc_command->ret_value);
@@ -805,9 +870,22 @@ gaim_plugins_probe(const char *ext)
 	GaimPlugin *plugin;
 	size_t i;
 
+	void *handle;
+
 	if (!g_module_supported())
 		return;
 
+	handle = gaim_plugins_get_handle();
+
+	gaim_debug_info("plugins", "registering plugin-load signal\n");
+	gaim_signal_register(handle, "plugin-load", gaim_marshal_VOID__POINTER, NULL,
+			1, gaim_value_new(GAIM_TYPE_SUBTYPE, GAIM_SUBTYPE_PLUGIN));
+
+	gaim_debug_info("plugins", "registering plugin-unload signal\n");
+	gaim_signal_register(handle, "plugin-unload", gaim_marshal_VOID__POINTER, NULL,
+			1, gaim_value_new(GAIM_TYPE_SUBTYPE, GAIM_SUBTYPE_PLUGIN));
+
+
 	for (i = 0; i < search_path_count; i++) {
 		if (search_paths[i] == NULL)
 			continue;
@@ -861,7 +939,6 @@ gaim_plugins_probe(const char *ext)
 		}
 		else if (plugin->info->type == GAIM_PLUGIN_PROTOCOL)
 		{
-
 			/* We'll just load this right now. */
 			if (!gaim_plugin_load(plugin))
 			{
@@ -870,8 +947,7 @@ gaim_plugins_probe(const char *ext)
 				continue;
 			}
 
-			if (GAIM_PLUGIN_PROTOCOL_INFO(plugin)->protocol == GAIM_PROTO_ICQ ||
-				gaim_find_prpl(GAIM_PLUGIN_PROTOCOL_INFO(plugin)->protocol))
+			if (gaim_find_prpl(plugin->info->id))
 			{
 				/* Nothing to see here--move along, move along */
 				gaim_plugin_destroy(plugin);
@@ -904,10 +980,34 @@ gaim_plugin_register(GaimPlugin *plugin)
 	if (g_list_find(plugins, plugin))
 		return TRUE;
 
-	if (plugin->info->type == GAIM_PLUGIN_LOADER ||
-		plugin->info->type == GAIM_PLUGIN_PROTOCOL)
+	if (plugin->info->type == GAIM_PLUGIN_LOADER)
 	{
-		/* Special exception for loader plugins. We want them loaded NOW! */
+		GaimPluginLoaderInfo *loader_info;
+
+		loader_info = GAIM_PLUGIN_LOADER_INFO(plugin);
+
+		if (loader_info == NULL)
+		{
+			gaim_debug(GAIM_DEBUG_ERROR, "plugins", "%s is unloadable\n",
+							   plugin->path);
+			return FALSE;
+		}
+
+		load_queue = g_list_append(load_queue, plugin);
+	}
+	else if (plugin->info->type == GAIM_PLUGIN_PROTOCOL)
+	{
+		GaimPluginProtocolInfo *prpl_info;
+
+		prpl_info = GAIM_PLUGIN_PROTOCOL_INFO(plugin);
+
+		if (prpl_info == NULL)
+		{
+			gaim_debug(GAIM_DEBUG_ERROR, "plugins", "%s is unloadable\n",
+							   plugin->path);
+			return FALSE;
+		}
+
 		load_queue = g_list_append(load_queue, plugin);
 	}
 
@@ -1072,3 +1172,15 @@ gaim_plugins_get_all(void)
 	return plugins;
 }
 
+
+GaimPluginAction *
+gaim_plugin_action_new(char* label, void (*callback)(GaimPluginAction *))
+{
+	GaimPluginAction *act = g_new0(GaimPluginAction, 1);
+
+	act->label = label;
+	act->callback = callback;
+
+	return act;
+}
+
