@@ -206,7 +206,7 @@ static void irc_login(GaimAccount *account)
 {
 	GaimConnection *gc;
 	struct irc_conn *irc;
-	char *buf, **userparts;
+	char **userparts;
 	const char *username = gaim_account_get_username(account);
 	int err;
 
@@ -233,9 +233,7 @@ static void irc_login(GaimAccount *account)
 	irc->msgs = g_hash_table_new(g_str_hash, g_str_equal);
 	irc_msg_table_build(irc);
 
-	buf = g_strdup_printf(_("Signon: %s"), username);
-	gaim_connection_update_progress(gc, buf, 1, 2);
-	g_free(buf);
+	gaim_connection_update_progress(gc, _("Connecting"), 1, 2);
 
 	err = gaim_proxy_connect(account, irc->server, 
 				 gaim_account_get_int(account, "port", IRC_DEFAULT_PORT),
@@ -316,6 +314,7 @@ static void irc_close(GaimConnection *gc)
 		gaim_timeout_remove(irc->timer);
 	g_hash_table_destroy(irc->cmds);
 	g_hash_table_destroy(irc->msgs);
+	g_hash_table_destroy(irc->buddies);
 	if (irc->motd)
 		g_string_free(irc->motd, TRUE);
 	g_free(irc->server);
@@ -340,8 +339,9 @@ static int irc_im_send(GaimConnection *gc, const char *who, const char *what, Ga
 static void irc_get_info(GaimConnection *gc, const char *who)
 {
 	struct irc_conn *irc = gc->proto_data;
-	const char *args[1];
+	const char *args[2];
 	args[0] = who;
+	args[1] = NULL;
 	irc_cmd_whois(irc, "whois", NULL, args);
 }
 
@@ -686,7 +686,7 @@ static void _init_plugin(GaimPlugin *plugin)
 	option = gaim_account_option_int_new(_("Port"), "port", IRC_DEFAULT_PORT);
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
 
-	option = gaim_account_option_string_new(_("Encoding"), "encoding", IRC_DEFAULT_CHARSET);
+	option = gaim_account_option_string_new(_("Encodings"), "encoding", IRC_DEFAULT_CHARSET);
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options, option);
 
 	option = gaim_account_option_string_new(_("Username"), "username", "");
