@@ -47,10 +47,6 @@
 #include "imgstore.h"
 #endif
 
-#define BUDDY_ALIAS_MAXLEN 387
-
-static GaimPlugin *my_protocol = NULL;
-
 typedef struct
 {
 	GaimConnection *gc;
@@ -107,7 +103,10 @@ msn_act_id(GaimConnection *gc, const char *entry)
 	cmdproc = session->notification->cmdproc;
 	account = gaim_connection_get_account(gc);
 
-	alias = (entry && *entry) ? entry : "";
+	if (entry && strlen(entry))
+		alias = gaim_url_encode(entry);
+	else
+		alias = "";
 
 	if (strlen(alias) > BUDDY_ALIAS_MAXLEN)
 	{
@@ -118,7 +117,7 @@ msn_act_id(GaimConnection *gc, const char *entry)
 
 	msn_cmdproc_send(cmdproc, "REA", "%s %s",
 					 gaim_account_get_username(account),
-					 gaim_url_encode(alias));
+					 alias);
 }
 
 static void
@@ -189,7 +188,7 @@ send_to_mobile(GaimConnection *gc, const char *who, const char *entry)
 
 	payload = msn_page_gen_payload(page, &payload_len);
 
-	trans = msn_transaction_new("PGD", "%s 1 %d", who, payload_len);
+	trans = msn_transaction_new(cmdproc, "PGD", "%s 1 %d", who, payload_len);
 
 	msn_transaction_set_payload(trans, payload, payload_len);
 
@@ -348,9 +347,34 @@ static void
 t_msn_xfer_init(GaimXfer *xfer)
 {
 	MsnSlpLink *slplink;
+	const char *filename;
+	FILE *fp;
+
+	filename = gaim_xfer_get_local_filename(xfer);
 
 	slplink = xfer->data;
 
+	if ((fp = g_fopen(filename, "rb")) == NULL)
+	{
+		GaimAccount *account;
+		GaimConnection *gc;
+		const char *who;
+		char *msg;
+
+		account = slplink->session->account;
+		gc = gaim_account_get_connection(account);
+		who = slplink->remote_user;
+
+		msg = g_strdup_printf(_("Error reading %s: \n%s.\n"),
+							  filename, strerror(errno));
+		gaim_xfer_error(gaim_xfer_get_type(xfer), who, msg);
+		gaim_xfer_cancel_local(xfer);
+		g_free(msg);
+
+		return;
+	}
+	fclose(fp);
+
 	msn_slplink_request_ft(slplink, xfer);
 }
 
@@ -377,6 +401,23 @@ msn_send_file(GaimConnection *gc, const char *who, const char *file)
 		gaim_xfer_request(xfer);
 }
 
+static gboolean
+msn_can_receive_file(GaimConnection *gc, const char *who)
+{
+	GaimAccount *account;
+	char *normal;
+	gboolean ret;
+
+	account = gaim_connection_get_account(gc);
+
+	normal = g_strdup(msn_normalize(account, gaim_account_get_username(account)));
+	ret = strcmp(normal, msn_normalize(account, who));
+
+	g_free(normal);
+
+	return ret;
+}
+
 /**************************************************************************
  * Protocol Plugin ops
  **************************************************************************/
@@ -409,8 +450,15 @@ msn_list_emblems(GaimBuddy *b, char **se, char **sw,
 	{
 		emblems[0] = "offline";
 	}
-	else if (user->mobile)
-		emblems[i++] = "wireless";
+	else
+	{
+		if (user->mobile)
+			emblems[i++] = "wireless";
+		if (user->list_op & (1 << MSN_LIST_BL))
+			emblems[i++] = "blocked";
+		if (!(user->list_op & (1 << MSN_LIST_RL)))
+			emblems[i++] = "nr";
+	}
 
 	*se = emblems[0];
 	*sw = emblems[1];
@@ -443,9 +491,15 @@ msn_tooltip_text(GaimBuddy *b)
 
 	user = b->proto_data;
 
-	g_string_append_printf(s, _("\n<b>%s:</b> %s"), _("Has you"),
-						   (user->list_op & (1 << MSN_LIST_RL)) ?
-						   _("Yes") : _("No"));
+	if (user != NULL)
+	{
+		g_string_append_printf(s, _("\n<b>%s:</b> %s"), _("Has you"),
+							   (user->list_op & (1 << MSN_LIST_RL)) ?
+							   _("Yes") : _("No"));
+		g_string_append_printf(s, _("\n<b>%s:</b> %s"), _("Blocked"),
+							   (user->list_op & (1 << MSN_LIST_BL)) ?
+							   _("Yes") : _("No"));
+	}
 
 	return g_string_free(s, FALSE);
 }
@@ -523,13 +577,12 @@ msn_buddy_menu(GaimBuddy *buddy)
 											 show_send_to_mobile_cb, NULL);
 			m = g_list_append(m, act);
 		}
-
 	}
 
 	if (g_ascii_strcasecmp(buddy->name,
 						   gaim_account_get_username(buddy->account)))
 	{
-		act = gaim_blist_node_action_new(_("Initiate Chat"),
+		act = gaim_blist_node_action_new(_("Initiate _Chat"),
 										 initiate_chat_cb, NULL);
 		m = g_list_append(m, act);
 	}
@@ -573,38 +626,27 @@ msn_login(GaimAccount *account)
 	}
 
 	if (gaim_account_get_bool(account, "http_method", FALSE))
-	{
 		http_method = TRUE;
 
-		gaim_debug_info("msn", "using http method\n");
+	host = gaim_account_get_string(account, "server", MSN_SERVER);
+	port = gaim_account_get_int(account, "port", MSN_PORT);
 
-		host = "gateway.messenger.hotmail.com";
-		port = 80;
-	}
-	else
-	{
-		host = gaim_account_get_string(account, "server", MSN_SERVER);
-		port = gaim_account_get_int(account,    "port",   MSN_PORT);
-	}
-
-	session = msn_session_new(account, host, port, http_method);
-	session->prpl = my_protocol;
-
-	if (session->http_method)
-		msn_http_session_init(session);
+	session = msn_session_new(account);
 
 	gc->proto_data = session;
 	gc->flags |= GAIM_CONNECTION_HTML | GAIM_CONNECTION_FORMATTING_WBFO | GAIM_CONNECTION_NO_BGCOLOR | GAIM_CONNECTION_NO_FONTSIZE | GAIM_CONNECTION_NO_URLDESC;
 
-	gaim_connection_update_progress(gc, _("Connecting"), 0, MSN_CONNECT_STEPS);
+	msn_session_set_login_step(session, MSN_LOGIN_STEP_START);
 
 	/* Hmm, I don't like this. */
+	/* XXX shx: Me neither */
 	username = msn_normalize(account, gaim_account_get_username(account));
 
 	if (strcmp(username, gaim_account_get_username(account)))
 		gaim_account_set_username(account, username);
 
-	msn_session_connect(session);
+	if (!msn_session_connect(session, host, port, http_method))
+		gaim_connection_error(gc, _("Failed to connect to server."));
 }
 
 static void
@@ -616,9 +658,6 @@ msn_close(GaimConnection *gc)
 
 	g_return_if_fail(session != NULL);
 
-	if (session->http_method)
-		msn_http_session_uninit(session);
-
 	msn_session_destroy(session);
 
 	gc->proto_data = NULL;
@@ -629,51 +668,65 @@ msn_send_im(GaimConnection *gc, const char *who, const char *message,
 			GaimConvImFlags flags)
 {
 	GaimAccount *account;
+	MsnMessage *msg;
+	char *msgformat;
+	char *msgtext;
 
 	account = gaim_connection_get_account(gc);
 
+	msn_import_html(message, &msgformat, &msgtext);
+
+	if (strlen(msgtext) + strlen(msgformat) + strlen(VERSION) > 1564)
+	{
+		g_free(msgformat);
+		g_free(msgtext);
+
+		return -E2BIG;
+	}
+
+	msg = msn_message_new_plain(msgtext);
+	msn_message_set_attr(msg, "X-MMS-IM-Format", msgformat);
+
+	g_free(msgformat);
+	g_free(msgtext);
+
 	if (g_ascii_strcasecmp(who, gaim_account_get_username(account)))
 	{
 		MsnSession *session;
 		MsnSwitchBoard *swboard;
-		MsnMessage *msg;
-		char *msgformat;
-		char *msgtext;
 
 		session = gc->proto_data;
 		swboard = msn_session_get_swboard(session, who);
 
-		msn_import_html(message, &msgformat, &msgtext);
-		msg = msn_message_new_plain(msgtext);
-		msn_message_set_attr(msg, "X-MMS-IM-Format", msgformat);
-
-		g_free(msgformat);
-		g_free(msgtext);
-
-		swboard = msn_session_get_swboard(session, who);
-
-		if (!g_queue_is_empty(swboard->im_queue) ||
-			!swboard->user_joined)
-		{
-			msn_switchboard_queue_msg(swboard, msg);
-		}
-		else
-		{
-			msn_switchboard_send_msg(swboard, msg);
-		}
-
-		msn_message_destroy(msg);
+		msn_switchboard_send_msg(swboard, msg, TRUE);
 	}
 	else
 	{
+		char *body_str, *body_enc, *pre, *post;
+		const char *format;
 		/*
 		 * In MSN, you can't send messages to yourself, so
 		 * we'll fake like we received it ;)
 		 */
+		body_str = msn_message_to_string(msg);
+		body_enc = gaim_escape_html(body_str);
+		g_free(body_str);
+
+		format = msn_message_get_attr(msg, "X-MMS-IM-Format");
+		msn_parse_format(format, &pre, &post);
+		body_str = g_strdup_printf("%s%s%s", pre ? pre :  "",
+								   body_enc ? body_enc : "", post ? post : "");
+		g_free(body_enc);
+		g_free(pre);
+		g_free(post);
+
 		serv_got_typing_stopped(gc, who);
-		serv_got_im(gc, who, message, flags, time(NULL));
+		serv_got_im(gc, who, body_str, flags, time(NULL));
+		g_free(body_str);
 	}
 
+	msn_message_destroy(msg);
+
 	return 1;
 }
 
@@ -701,20 +754,17 @@ msn_send_typing(GaimConnection *gc, const char *who, int typing)
 
 	swboard = msn_session_find_swboard(session, who);
 
-	if (swboard == NULL)
+	if (swboard == NULL || !msn_switchboard_can_send(swboard))
 		return 0;
 
-	if (!swboard->user_joined)
-		return 0;
-
-	msg = msn_message_new();
+	msg = msn_message_new(MSN_MSG_TYPING);
 	msn_message_set_content_type(msg, "text/x-msmsgscontrol");
 	msn_message_set_flag(msg, 'U');
 	msn_message_set_attr(msg, "TypingUser",
 						 gaim_account_get_username(account));
 	msn_message_set_bin_data(msg, "\r\n", 2);
 
-	msn_switchboard_send_msg(swboard, msg);
+	msn_switchboard_send_msg(swboard, msg, FALSE);
 
 	msn_message_destroy(msg);
 
@@ -1002,16 +1052,21 @@ msn_chat_invite(GaimConnection *gc, int id, const char *msg,
 {
 	MsnSession *session;
 	MsnSwitchBoard *swboard;
-	MsnCmdProc *cmdproc;
 
 	session = gc->proto_data;
 
-	swboard = msn_session_find_switch_with_id(session, id);
-	g_return_if_fail(swboard != NULL);
+	swboard = msn_session_find_swboard_with_id(session, id);
 
-	cmdproc = swboard->servconn->cmdproc;
+	if (swboard == NULL)
+	{
+		/* if we have no switchboard, everyone else left the chat already */
+		swboard = msn_switchboard_new(session);
+		msn_switchboard_request(swboard);
+		swboard->chat_id = id;
+		swboard->conv = gaim_find_chat(gc, id);
+	}
 
-	msn_cmdproc_send(cmdproc, "CAL", "%s", who);
+	msn_switchboard_request_add_user(swboard, who);
 }
 
 static void
@@ -1019,19 +1074,27 @@ msn_chat_leave(GaimConnection *gc, int id)
 {
 	MsnSession *session;
 	MsnSwitchBoard *swboard;
-	MsnCmdProc *cmdproc;
+	GaimConversation *conv;
 
 	session = gc->proto_data;
 
-	swboard = msn_session_find_switch_with_id(session, id);
-	g_return_if_fail(swboard != NULL);
+	swboard = msn_session_find_swboard_with_id(session, id);
 
-	cmdproc = swboard->servconn->cmdproc;
+	/* if swboard is NULL we were the only person left anyway */
+	if (swboard == NULL)
+		return;
 
-	msn_cmdproc_send_quick(cmdproc, "OUT", NULL, NULL);
-	msn_switchboard_destroy(swboard);
+	conv = swboard->conv;
 
-	/* serv_got_chat_left(gc, id); */
+	msn_switchboard_close(swboard);
+
+	/* If other switchboards managed to associate themselves with this
+	 * conv, make sure they know it's gone! */
+	if (conv != NULL)
+	{
+		while ((swboard = msn_session_find_swboard_with_conv(session, conv)) != NULL)
+			swboard->conv = NULL;
+	}
 }
 
 static int
@@ -1046,27 +1109,35 @@ msn_chat_send(GaimConnection *gc, int id, const char *message)
 
 	account = gaim_connection_get_account(gc);
 	session = gc->proto_data;
-	swboard = msn_session_find_switch_with_id(session, id);
+	swboard = msn_session_find_swboard_with_id(session, id);
 
 	if (swboard == NULL)
 		return -EINVAL;
 
-	if (swboard->ready)
+	if (!swboard->ready)
+		return 0;
+
+	msn_import_html(message, &msgformat, &msgtext);
+
+	if (strlen(msgtext) + strlen(msgformat) + strlen(VERSION) > 1564)
 	{
-		msn_import_html(message, &msgformat, &msgtext);
-
-		msg = msn_message_new_plain(msgtext);
-		msn_message_set_attr(msg, "X-MMS-IM-Format", msgformat);
-		msn_switchboard_send_msg(swboard, msg);
-		msn_message_destroy(msg);
-
 		g_free(msgformat);
 		g_free(msgtext);
 
-		serv_got_chat_in(gc, id, gaim_account_get_username(account), 0,
-						 message, time(NULL));
+		return -E2BIG;
 	}
 
+	msg = msn_message_new_plain(msgtext);
+	msn_message_set_attr(msg, "X-MMS-IM-Format", msgformat);
+	msn_switchboard_send_msg(swboard, msg, FALSE);
+	msn_message_destroy(msg);
+
+	g_free(msgformat);
+	g_free(msgtext);
+
+	serv_got_chat_in(gc, id, gaim_account_get_username(account), 0,
+					 message, time(NULL));
+
 	return 0;
 }
 
@@ -1074,13 +1145,17 @@ static void
 msn_keepalive(GaimConnection *gc)
 {
 	MsnSession *session;
-	MsnCmdProc *cmdproc;
 
 	session = gc->proto_data;
-	cmdproc = session->notification->cmdproc;
 
 	if (!session->http_method)
+	{
+		MsnCmdProc *cmdproc;
+
+		cmdproc = session->notification->cmdproc;
+
 		msn_cmdproc_send_quick(cmdproc, "PNG", NULL, NULL);
+	}
 }
 
 static void
@@ -1127,33 +1202,33 @@ msn_convo_closed(GaimConnection *gc, const char *who)
 {
 	MsnSession *session;
 	MsnSwitchBoard *swboard;
-	MsnCmdProc *cmdproc;
+	GaimConversation *conv;
 
 	session = gc->proto_data;
 
 	swboard = msn_session_find_swboard(session, who);
 
 	/*
-	 * Don't perform an assertion here. It swboard is NULL, then the
+	 * Don't perform an assertion here. If swboard is NULL, then the
 	 * switchboard was either closed by the other party, or the person
 	 * is talking to himself.
 	 */
 	if (swboard == NULL)
 		return;
 
-	cmdproc = swboard->servconn->cmdproc;
+	conv = swboard->conv;
 
-	if (swboard->current_users == 1)
+	if (!(swboard->flag & MSN_SB_FLAG_FT))
+		msn_switchboard_close(swboard);
+	else
+		swboard->conv = NULL;
+
+	/* If other switchboards managed to associate themselves with this
+	 * conv, make sure they know it's gone! */
+	if (conv != NULL)
 	{
-		/* This must happen on both IM's and Chat's, right? */
-		GaimAccount *account;
-
-		account = gaim_connection_get_account(gc);
-
-		msn_cmdproc_send_quick(cmdproc, "BYE", "%s",
-							   gaim_account_get_username(account));
-
-		msn_switchboard_destroy(swboard);
+		while ((swboard = msn_session_find_swboard_with_conv(session, conv)) != NULL)
+			swboard->conv = NULL;
 	}
 }
 
@@ -1391,7 +1466,7 @@ msn_got_info(void *data, const char *url_text, size_t len)
 
 	/* Extract their Name and put it in */
 	found = gaim_markup_extract_info_field(stripped, stripped_len, s,
-			"\tName\n", 0, "\t", 0, "Undisclosed", _("Name"), 0, NULL);
+			"\nName\n", 0, "\t", 0, "Undisclosed", _("Name"), 0, NULL);
 
 	if (found)
 		has_info = TRUE;
@@ -1783,7 +1858,7 @@ static GaimPluginProtocolInfo prpl_info =
 	NULL,					/* warn */
 	NULL,					/* join_chat */
 	NULL,					/* reject chat invite */
-	NULL,				/* get_chat_name */
+	NULL,					/* get_chat_name */
 	msn_chat_invite,		/* chat_invite */
 	msn_chat_leave,			/* chat_leave */
 	NULL,					/* chat_whisper */
@@ -1806,7 +1881,7 @@ static GaimPluginProtocolInfo prpl_info =
 	NULL,					/* roomlist_get_list */
 	NULL,					/* roomlist_cancel */
 	NULL,					/* roomlist_expand_category */
-	NULL,					/* can_receive_file */
+	msn_can_receive_file,	/* can_receive_file */
 	msn_send_file			/* send_file */
 };
 
@@ -1860,8 +1935,6 @@ init_plugin(GaimPlugin *plugin)
 	prpl_info.protocol_options = g_list_append(prpl_info.protocol_options,
 											   option);
 
-	my_protocol = plugin;
-
 	gaim_prefs_add_none("/plugins/prpl/msn");
 	gaim_prefs_add_bool("/plugins/prpl/msn/conv_close_notice",   TRUE);
 	gaim_prefs_add_bool("/plugins/prpl/msn/conv_timeout_notice", TRUE);
