@@ -104,6 +104,8 @@ destroy_signal_data(GaimSignalData *signal_data)
 		g_free(signal_data->values);
 	}
 
+	if (signal_data->ret_value != NULL)
+		gaim_value_destroy(signal_data->ret_value);
 	g_free(signal_data);
 }
 
